# shadow-device

## Data structure

```C
enum {INT, STR, ARR} TAG;
```

 TAG is attr tag.

**Members**

* INT: int type.
* STR: string type.
* ARR: attr_in type.

---

```C
enum {REPORTED, DESIRED} emqc_ds_state;
```

State is a tag to mark properties.

**Members**

* reported: reported state.
* desired: desired state.

---

```C
struct attr_in {
    TAG                 tag;
    union {
        int             val_int;
        char            *val_str;
        struct attr     *val_arr;
    };
} attr_in;
```

Attr_in is used to store array type.

**Members**

* tag: To decide value type we will store.
* val_int: int value.
* val_str: string value.
* val_arr: array value.

---

```c
struct attr {
    TAG         tag;
    char        key;
    union {
        int     val_int;
        char    *val_str;
        attr    *val_arr;
    };
};
```

Attr is a key value struct, it can store key value pairs parse from json.

**Members**

* tag: To decide value type we will store.
* key: key.
* val_int: int value.
* val_str: string value.
* val_arr: array value.

---



## API 



```C
emqc_ds *emqc_ds_new();
```

Create a new emqc device shadow client instance.  

**Returns**

 *  emqc client instance.

```C
EMQC_ERR_CODE emqc_ds_init((emqc_ds *handle, char *host, int port);
```

Initialize emqc client instance.  

**Parameters**

 *  handle: emqc device shadow handle.
 *  host : hostname or ip address of the broker to connect to.  
 *  port : network port to connect to.  Usually 1883.

**Returns**

 *  error code.

---

```C
void emqc_ds_destroy(emqc_ds *handle);
```

Use to free memory associated with a emqc client instance.  

**Parameters**

 *  handle : emqc client instance.  

**Returns**

---

```C
err_code emqc_ds_connect(emqc_ds *handle, int keep_alive_interval, int clean_session);
```

Connect to an mqtt broker.  

**Parameters**

 *  handle : an emqc device shadow client instance.  
 *  keep_alive_interval : keep alive interval.
 *  clean_session : clean session.

**Returns**

* error code.

---

```C
void emqc_ds_set_host(emqc_ds *handle, char *host);
```

Set host.

**Parameters**

* handle : an emqc device shadow client instance.

* host : host.

**Returns**

---

```C
void emqc_ds_set_port(emqc_ds *handle, int port);
```

Set port.

**Parameters**

 *  handle : an emqc device shadow client instance.
 *  port : port.

---

```C
err_code emqc_ds_disconnect(emqc_ds *handle);
```

Disconnect from a broker.  

**Parameters**

 *  handle : an emqc device shadow client instance. 

**Returns**

* error code.

---

```C
void emqc_ds_set_get_callback(emqc_ds *handle, callback func);
```

Set get callback.  

**Parameters**

 *  handle : an emqc device shadow client instance.
 *  func : a callback function for receive data.

**Returns**

```C
void emqc_ds_set_control_callback(emqc_ds *handle, callback func);
```

Set control callback.  

**Parameters**

 *  handle : an emqc device shadow client instance.
 *  func : a callback function for receive data.

**Returns**

---

```C
void emqc_ds_set_generic_callback(emqc_ds *handle, callback func);
```

Set generic callback.  

**Parameters**

 *  handle : an emqc device shadow client instance.
 *  func : a callback function for update or delete data.

**Returns**

---

```C
EMQC_ERR_CODE emqc_ds_asyn_get(emqc_ds *handle);
```

Get information from shadow.  

**Parameters**

 *  handle : an emqc device shadow client instance.

**Returns**

* error code.

**Returns**

---

```C
EMQC_ERR_CODE emqc_ds_asyn_update(emqc_ds *handle, serv_req *resquest);
```

Update information to shadow.  

**Parameters**

 *  handle : an emqc device shadow client instance.
 *  request : server request.

**Returns**

* error code.

---

```C
EMQC_ERR_CODE emqc_ds_asyn_delete(emqc_ds *handle, serv_req *request);
```

Delete shadow data.  

**Parameters**

 *  handle : an emqc client instance.
 *  request : server request.

**Returns**

* error code.

---

```C
serv_req *new_request(void);
```

New server_request object.

**Parameters**

* Void.

**Returns**

* server_request object.

---

```C
void free_request(server_request *request);
```

Free memory server_request object.

**Parameters**

 * request : server_request object.

**Returns**

---

```C
void free_attr_vec(attr **attr_);
```

Free memory of all attributes.

**Parameters**

 *  attr : attributes of array.

**Returns**

---

```C
void set_request_version(serv_req *request, long version);
```

Set request current version.

**Parameters**

 *  request : server request.
 *  version : current version.

**Returns**

---

```C
long get_version(serv_resp *re);
```

Get current version from response.

**Parameters**

 *  re : server response
 *  ret : version

**Returns**

* Current version.

---

```C
const char *get_param_method(serv_resp *response);
```

Get method from server response

**Parameters**

 *  response : server response

**Returns**

* method.

---

```C
attr **get_all_param_attr(serv_resp *response, emqc_ds_state s);
```

Get all type and structure with key && value

**Parameters**

 *  response : server response
 *  s : desired or reported

**Returns**

* attribute int array.

---

```C
int add_param_null(serv_req *request, char *key, emqc_ds_state s);
```

Add null to server request for delete

**Parameters**

 * request : server request
 * key : key
 * s : state desired or reported

**Returns**

---

```C
int add_param_int(serv_req *request, char *key, int val, emqc_ds_state s);
```

Add int type to server request

**Parameters**

 * request : server request
 * key : key
 * val : value
 * s : state desired or reported

**Returns**

---

```C
int add_param_str(server_request *request, char *key, char *val, emqc_ds_state s);
```

Add string type to server request

**Parameters**

 * request : server request
 * key : key
 * val : value
 * s : state desired or reported

**Returns**

---

```C
int add_param_arr(serv_req *request, const char *key, void *val, TAG type, int len, emqc_ds_state s);
```

Add array type to server request.

**Parameters**

 * request : server request
 * key : key
 * val : value array
 * TAG: array type
 * len : value length
 * s : state desired or reported

**Returns**
