# emqc-base

## Data structure

---

```C
enum {
    EMQC_SUCCESS,
    EMQC_CONN_ERR,
    EMQC_DISCONN_ERR,
    EMQC_GET_ERR,
    EMQC_UPDATE_ERR,
    EMQC_DELETE_ERR,
    EMQC_FILED_MISSING_ERR,
    EMQC_INVALID_JSON_ERR,
    EMQC_SUBSCRIBE_FAILED_ERR,
    EMQC_PUBLISH_FAILED_ERR,
    EMQC_MEMORY_ALLOC_FAILED_ERR,
    EMQC_CLIENT_CREATE_ERR,
} EMQC_ERR_CODE;
```

Err_code is uniform return status.

**Members**

- EMQC_SUCCESS: success.
- EMQC_CONN_ERR: connection error.
- EMQC_DISCONN_ERR: disconnection error.
- EMQC_GET_ERR: get error.
- EMQC_UPDATE_ERR: update error.
- EMQC_DELETE_ERR: delete error.
- EMQC_FILED_MISSING_ERR: device_name or product_key is missing.

- EMQC_INVALID_JSON_ERR: invalid json data.
- EMQC_SUBSCRIBE_FAILED_ERR: subscribe failed.
- EMQC_PUBLISH_FAILED_ERR: publish failed.
- EMQC_MEMORY_ALLOC_FAILED_ERR: "memory alloc failed.
- EMQC_CLIENT_CREATE_ERR: client create failed.

---

```C
struct {
    char *     product_key;
    char *     device_name;
    int        reconn_interval;
    int        reconn_times;
    Client *   client;
    Conn_opts *conn_opts;
} emqc;
```

Emqc client instance.  

**Members**

* product_key: product key.
* device_name: device name.
* reconn_interval: reconnect interval.
* reconn_times: reconnect times.
* client: mqtt client.
* conn_opts: mqtt connect options.

---



## API 

```C
emqc *emqc_alloc();
```

Create a new emqc client instance.  

**Returns**

 *  emqc client instance.

```C
EMQC_ERR_CODE emqc_init((emqc *handle, char *host, int port);
```

Initialize emqc client instance.  

**Parameters**

 *  handle: emqc handle.
 *  host : hostname or ip address of the broker to connect to.  
 *  port : network port to connect to.  Usually 1883.

**Returns**

 *  error code.

---

```C
void emqc_destroy(emqc *handle);
```

Use to free memory associated with a emqc client instance.  

**Parameters**

 *  handle : emqc client instance.  

**Returns**

---

```C
err_code emqc_connect(emqc *handle, int keep_alive_interval, int clean_session);
```

Connect to an mqtt broker.  

**Parameters**

 *  handle : an emqc client instance.  
 *  keep_alive_interval : keep alive interval.
 *  clean_session : clean session.

**Returns**

* error code.

---

```C
void emqc_set_device_name(emqc *handle, char *device_name);
```

 Set device name.  

**Parameters**

 *   handle : emqc client instance.  
 *   device_name : device name.  

**Returns**

---

```C
void emqc_set_product_key(emqc *handle, char *product_key);
```

Set product key.

**Parameters**

* handle : emqc client instance.  
* product_key : product key.  

**Returns**

---

```C
void emqc_set_secret(emqc *handle, char *password);
```

Set secret.
**Parameters**

*  handle : an emqc client instance.
*  secert : secret.

**Returns**

---

```C
void emqc_set_host(emqc *handle, char *host);
```

Set host.

**Parameters**

* handle : an emqc client instance.

* host : host.

**Returns**

---

```C
void emqc_set_port(emqc *handle, int port);
```

Set port.

**Parameters**

 *  handle : an emqc client instance.
 *  port : port.

---

```C
EMQC_ERR_CODE emqc_disconnect(emqc *handle);
```

Disconnect from a broker.  

**Parameters**

 *  handle : an emqc client instance. 

**Returns**

* error code.
