# thing-model

## Data structure

```C
enum {
    TM_CB_PROPERTY_POST_REPLY,
    TM_CB_PROPERTY_SET,
    TM_CB_EVENT_POST_REPLY,
    TM_CB_ASYN_ACTION_EXEC,
    TM_CB_SYN_ACTION_EXEC,
    TM_CB_DESIRED_GET_REPLY,
    TM_CB_DESIRED_DEL_REPLY
} emqc_tm_cb_opt;
```

 emqc_tm_cb_opt  is a callback function identifer.

**Members**

* TM_CB_PROPERTY_POST_REPLY:  property post reply related callback function identifer.
* TM_CB_PROPERTY_SET:  property set related callback function identifer.
* TM_CB_EVENT_POST_REPLY: event post reply related callback function identifer.
* TM_CB_ASYN_ACTION_EXEC: asynchrononous action exec related callback function identifer.
* TM_CB_SYN_ACTION_EXEC: synchrononous action exec related callback function identifer.
* TM_CB_DESIRED_GET_REPLY: desired get reply related callback function identifer.
* TM_CB_DESIRED_DEL_REPLY: desired delete reply related callback function identifer.

---

```C
enum {
    EMQC_TM_INT,
    EMQC_TM_STR,
    EMQC_TM_DOUBLE,
    EMQC_TM_STRUCT,
    EMQC_TM_ARR_INT,
    EMQC_TM_ARR_STR,
    EMQC_TM_ARR_DOUBLE,
    EMQC_TM_ARR_STRUCT,
    EMQC_TM_TERMINATOR,
} emqc_tm_type;
```

Emqc_tm_type is a type identifer.

**Members**

* EMQC_TM_INT: int type identifer.
* EMQC_TM_STR: string type identifer.
* EMQC_TM_DOUBLE: double type identifer.
* EMQC_TM_STRUCT: struct type identifer.
* EMQC_TM_ARR_INT: int type array identifer.
* .EMQC_TM_ARR_STR: string type array identifer.
* EMQC_TM_ARR_DOUBLE: double type array identifer.
* EMQC_TM_ARR_STRUCT: struct type array identifer.
* EMQC_TM_TERMINATOR: terminator identifer use for array terminator.

---

```C
struct {
    emqc_tm_type    type;
    char            *key;
    union {
        int         val_i;
        double      val_d;
        char        *val_s;
        void        *val_o;
    };

} emqc_tm_object;
```

Emqc_tm_object is an object to manage data you want to transfer .

**Members**

* type: type identifer.
* key: json key data.
* val_i: int type value.
* val_d: double type value.
* val_s: string type value.
* val_o: array or struct type value .

---

```C
struct {
    char *id;
    char *data;
    char *msg;
    int  code;
} tm_reply;
```

Tm_reply is used to get reply from server reply payload.

**Members**

* id: message id number.
* data: while request is success, data is NULL.
* msg: while request is failed, error message will store here.
* code: status code.

---

```c
struct {
    char *id;
    char *version;
    char *method;
    char *params;
} tm_request;
```

Tm_request is used to get request from server request payload.

**Members**

* id: message id number.
* version: protocol version.
* method: request method.
* params: request params.

---

```c
struct {
    enum {
      TM_REP, 
      TM_REQ, 
      TM_RAW
    } type;
    union {
        tm_reply *rep;
        tm_request *req;
        char     *raw;
    };
} emqc_tm_trans_object;
```

emqc_tm_trans_object is used to get request from server  payload.

**Members**

* type: type contain TM_REP for reply payload, TM_REQ for request payload, raw for raw data.
* rep: reply data.
* req: request data.
* raw: raw data, for customiz data.

---

```C
struct {
    emqc *    handle;
    tm_ctxt   **ctxts;
} emqc_tm;
```

Emqc_tm is thing model handle.

**Members**

- handle: emqc handle.
- ctxts：ctxts contain all subscribe topic && callback function.  

---

```C
struct {
    char *key;
    char *version;
} del_des_param;
```

Del_des_param is 

**Members**

- key: 
- version:

## API 



```C
emqc_tm *emqc_tm_new();
```

Create a new emqc thing model client instance.  

**Returns**

 *  emqc thing model client instance.

```C
emqc_tm *emqc_init((emqc *tm, const char *host, int port);
```

Initialize emqc thing model client instance.  

**Parameters**

 *  tm: emqc thing model handle.
 *  host : hostname or ip address of the broker to connect to.  
 *  port : network port to connect to.  Usually 1883.

**Returns**

 *  emqc_tm handle.

---

```C
void emqc_tm_destroy(emqc_tm *tm);
```

Use to free memory associated with a emqc thing model client instance.  

**Parameters**

 *  tm : emqc thing model client instance.  

**Returns**

---

```C
int emqc_tm_connect(emqc_tm *handle);
```

Connect to an mqtt broker.  

**Parameters**

 *  handle : an emqc client instance.  

**Returns**

* error code

---

```C
void emqc_tm_set_dft_cb(emqc_tm *tm, emqc_tm_cb_opt opt, emqc_cb cb);
```

 Set default callback .  

**Parameters**

 *   tm: emqc thing model client instance.  
 *   opt : callback function identifer.
 *   cb: callback function.

**Returns**

---

```C
void emqc_tm_set_ctm_cb(emqc_tm *tm, emqc_tm_cb_opt opt, emqc_cb cb, char *mid, char *identifier);
```

 Set costom callback .  

**Parameters**

 *   tm: emqc thing model client instance.  
 *   opt : callback function identifer.
 *   cb: callback function.
 *   mid: model identifer.
 *   identifer: function identifer.

**Returns**

---

```C
EMQC_ERR_CODE emqc_tm_sub(emqc_tm *tm);
```

Subscribe all topic, we have set.
**Parameters**

*  tm : an emqc thing model handle.

**Returns**

---

```C
EMQC_ERR_CODE emqc_tm_pub(emqc_tm *tm, const char *topic, const char *payload);

```

Publish thing model.

**Parameters**

* handle : an emqc thing model handle.
* topic: topic.
* payload: payload.

**Returns**

- error code.

---

```C
json_object *emqc_tm_new_payload(const char *mtd_dft, const char *func_id);
```

New publish payload.

**Parameters**

 *  mtd_dft : default method.
 *  func_id : function identifer.

**Returns**

- Publish json payload handle.

---

```C
json_object *emqc_tm_add_get_desired_param(json_object *jso, const char **keys, int num_keys, char *func_id);
```

Add get desired param to publish handle.  

**Parameters**

 *  jso : publish json payload handle.
 *  keys: array of key about desire you want to get.
 *  num_keys: number of keys.
 *  func_id:  function identifer.

**Returns**

* Publish json payload handle.

---

```C
json_object *emqc_tm_add_del_desired_param(json_object *jso, del_des_param *param_arr, int num_params, char *func_id);
```

  Add delete desired param to publish handle. 

**Parameters**

 *  jso: publish json payload handle.
 *  param_arr: del_des_param array about delete desired param.
 *  num_params: number of params.
 *  func_id:  function identifer.

**Returns**

- Publish json payload handle.

```C
json_object *emqc_tm_add_post_param(json_object *jso, emqc_tm_object *obj, char *func_id);
```

Add property post param to publish handle.  

**Parameters**

 *  jso : publish json payload handle.
 *  obj: emqc thing model object.
 *  func_id: function identifer.

**Returns**

- Publish json payload handle.

---

```C
json_object *emqc_tm_add_event_param(json_object *jso, emqc_tm_object *obj, bool ts);
```

Add event post param to publish handle.  

**Parameters**

 *  jso: publish json payload handle.
 *  obj: emqc thing model object.
 *  ts: if we need timestamp.

**Returns**

- Publish json payload handle.

---

```C
void emqc_tm_property_set(json_object *jso_tmp, emqc_tm_object property_array[], char *model_id);
```

Get information from shadow.  

**Parameters**

 *  jso_tmp: json parameter payload handle.
 *  property_array: emqc thing model object array about property
 *  func_id: function identifer.

**Returns**

