#include "device_shadow.h"
#include <unistd.h>
#include <zmalloc.h>

static long version = -1;

static void print_attrs(serv_resp *resp_, char *method)
{
    puts(method);
    serv_resp *resp = (serv_resp *) resp_;
    // TODO lock
    version = get_version(resp);
    printf("version: %ld\n", version++);

    char *payload = get_payload(resp);

    printf("payload: %s\n\n", payload);
    attr **v = get_all_param_attr(resp, REPORTED);
    puts("key  |  value  |  timestamp");
    puts("---------------------------");

    for (int i = 0; i < cvector_size(v); i++) {
        printf("%s ", v[i]->key);
        if (v[i]->tag == INT) {
            printf(" %d ", v[i]->val_int);
        } else if (v[i]->tag == STR) {
            printf(" %s ", v[i]->val_str);
        } else if (v[i]->tag == ARR) {
            for (int j = 0; j < cvector_size(v[i]->val_arr); j++) {
                if (v[i]->val_arr->tag == INT) {
                    printf(" %d ", v[i]->val_arr[j].val_int);
                } else if (v[i]->val_arr->tag == STR) {
                    printf(" %s ", v[i]->val_arr[j].val_str);
                }
            }
        } else {
            log_info("what");
        }
        printf(" %ld\n", v[i]->timestamp);
    }

    free_attr_vec(v);

    puts("---------------------------");
    v = get_all_param_attr(resp, DESIRED);

    for (int i = 0; i < cvector_size(v); i++) {
        printf("%s ", v[i]->key);
        if (v[i]->tag == INT) {
            printf(" %d ", v[i]->val_int);
        } else if (v[i]->tag == STR) {
            printf(" %s ", v[i]->val_str);
        } else if (v[i]->tag == ARR) {
            for (int j = 0; j < cvector_size(v[i]->val_arr); j++) {
                if (v[i]->val_arr->tag == INT) {
                    printf(" %d ", v[i]->val_arr[j].val_int);
                } else if (v[i]->val_arr->tag == STR) {
                    printf(" %s ", v[i]->val_arr[j].val_str);
                }
            }
        } else {
            log_info("what");
        }
        printf(" %ld\n", v[i]->timestamp);
    }

    free_attr_vec(v);
}

static int get_payload_call(void *resp_)
{
    print_attrs(resp_, "[get]:");
    return 0;
}

static int generic_payload_call(void *resp_)
{
    print_attrs(resp_, "[generic]:");
    return 0;
}

static int control_payload_call(void *resp_)
{
    print_attrs(resp_, "[control]:");
    return 0;
}

int main(int argc, char **argv)
{
    puts("===device shadow start===");

    int   interval = 5;
    int   times    = 10;
    int   rc;
    char *product_key = "KS79WXPI";
    char *device_name = "shadow";
    char *secret      = "9e4e784e66edf8a6ddd59ec05d7192b6";

    emqc_ds *ds = emqc_ds_new();

    emqc_set_secret(ds->handle, secret);
    emqc_set_product_key(ds->handle, product_key);
    emqc_set_device_name(ds->handle, device_name);

    emqc_ds_set_get_callback(ds, get_payload_call);
    emqc_ds_set_generic_callback(ds, generic_payload_call);
    emqc_ds_set_control_callback(ds, control_payload_call);

    emqc_set_reconnect(ds->handle, interval, times);

    emqc_ds_init(ds, "8.136.220.253", 1883);

    rc = emqc_ds_connect(ds, 60, 1);

    if (rc) {
        // err_code_info(rc);
    }

    for (;;) {

        serv_req *re_update = new_request();
        serv_req *re_delete = new_request();

        puts("\nSelect method: [delete], [update], [get]");
        char method[20];

        if (scanf("%s", method) != 1) {
            continue;
        }

        size_t size   = 1024;
        char * buffer = (char *) malloc(size * sizeof(char));

        if (buffer == NULL) {
            log_err("memory out!");
            break;
        }

        char key[20];
        // char val[30];
        char s[10];

        if (!strcmp(method, "delete")) {
            set_request_version(re_delete, version);
            puts("\ninput [key] [state] or [finish]");
            size_t nread = getline(&buffer, &size, stdin);
            if (nread == -1) {
                log_err("Read line failed");
            }

            for (;;) {
                nread = getline(&buffer, &size, stdin);
                if (nread == -1) {
                    log_err("Read line failed");
                }
                if (!strstr(buffer, "finish")) {
                    if (!sscanf(buffer, "%s %s", key, s)) {
                        puts("HELP");
                        // TODO
                    } else {
                        if (!strcmp(key, "null") || !strcmp(key, "*")) {
                            if (!strcmp(s, "reported")) {
                                add_param_null(re_delete, NULL, REPORTED);
                            } else if (!strcmp(s, "desired")) {
                                add_param_null(re_delete, NULL, DESIRED);
                            } else {
                                printf("state: %s", s);
                                puts("wrong input");
                            }
                        } else {
                            if (!strcmp(s, "reported")) {
                                add_param_null(re_delete, key, REPORTED);
                            } else if (!strcmp(s, "desired")) {
                                add_param_null(re_delete, key, DESIRED);
                            } else {
                                printf("state: %s", s);
                                puts("wrong input");
                            }
                        }
                    }

                } else {
                    if (!strcmp(s, "reported")) {
                        rc = emqc_ds_asyn_delete(ds, re_delete);
                    } else if (!strcmp(s, "desired")) {
                        rc = emqc_ds_asyn_update(ds, re_delete);
                    }

                    if (rc) {
                        // err_code_info(rc);
                    }
                    free_request(re_delete);
                    break;
                }
            }

        } else if (!strcmp(method, "update")) {
            set_request_version(re_update, version);

            puts("\ninput [key] [state] [type] [value] or [finish]");
            // puts("\ninput [key] [state] [type] [num] [value] or [finish]");
            size_t nread = getline(&buffer, &size, stdin);
            if (nread == -1) {
                log_err("Read line failed");
            }

            for (;;) {
                nread = getline(&buffer, &size, stdin);
                if (nread == -1) {
                    log_err("Read line failed");
                }
                if (!strstr(buffer, "finish")) {
                    // size_t num = 0;
                    char type[10];

                    // if (!sscanf(buffer,"%s %s %s %zd", key, s, type, &num)) {
                    if (!sscanf(buffer, "%s %s %s", key, s, type)) {
                        puts("HELP");
                        // TODO
                    } else {

                        emqc_ds_state st;

                        if (!strcmp("desired", s)) {
                            st = DESIRED;
                        } else if (!strcmp("reported", s)) {
                            st = REPORTED;
                        } else {
                            log_err("Invalid state!");
                            continue;
                        }

                        if (!strcmp(type, "STR_ARR")) {
                            char *t2[] = { "11", "22", "33", "44", "55" };
                            add_param_arr(re_update, key, t2, STR, 5, st);

                        } else if (!strcmp(type, "INT_ARR")) {
                            // int *t = (int*) zmalloc(sizeof(int*) * num);
                            // if (t == NULL) {
                            //     log_err("Memory alloc error!");
                            // }

                            // char nums[2];
                            // snprintf(nums, 2, "%zu", num);
                            //
                            // char *p = strstr(buffer, nums);
                            // char *q = strchr(p, ' ');
                            // while (q) {
                            //     p = strchr(q, ' ');
                            // }

                            int t1[] = { 1, 2, 3, 4, 5 };
                            add_param_arr(re_update, key, t1, INT, 5, st);
                            // zfree(t);

                        } else if (!strcmp(type, "STR")) {
                            char val[100];
                            if (sscanf(buffer, "%*s %*s %*s %s", val)) {
                                add_param_str(re_update, key, val, st);
                            } else {
                                log_err("Error value!");
                            }

                        } else if (!strcmp(type, "INT")) {
                            long val = 0;
                            if (sscanf(buffer, "%*s %*s %*s %ld", &val)) {
                                add_param_int(re_update, key, val, st);
                            } else {
                                log_err("Error value!");
                            }

                        } else {
                            log_err("Invalid type.");
                        }
                    }

                } else {
                    rc = emqc_ds_asyn_update(ds, re_update);
                    if (rc) {
                        // err_code_info(rc);
                    }
                    free_request(re_update);
                    break;
                }
            }

        } else if (!strcmp(method, "get")) {
            rc = emqc_ds_asyn_get(ds);
            if (rc) {
                // err_code_info(rc);
            }

        } else {
            puts("Wrong method!");
        }
    }

    rc = emqc_disconnect(ds->handle);
    if (rc) {
        // err_code_info(rc);
    }

    emqc_ds_destroy(ds);
    puts("===device shadow finish===");

    return 0;
}
