#ifndef THING_MODEL_H
#define THING_MODEL_H
#include <dbg.h>
#include <emqc_base.h>
#include <json-c/json.h>
#include <stdbool.h>

#define EMQC_TM_TOPIC_LEN 128
const char *TM_FMT = "/fabric/sys/%s/%s/thing/%s";

typedef enum {
    TM_CB_PROPERTY_POST_REPLY = 0,
    TM_CB_PROPERTY_SET,
    TM_CB_EVENT_POST_REPLY,
    TM_CB_ASYN_ACTION_EXEC,
    TM_CB_SYN_ACTION_EXEC,
    TM_CB_DESIRED_GET_REPLY,
    TM_CB_DESIRED_DEL_REPLY
} emqc_tm_cb_opt;

typedef enum {
    EMQC_TM_INT,
    EMQC_TM_STR,
    EMQC_TM_DOUBLE,
    EMQC_TM_STRUCT,
    EMQC_TM_ARR_INT,
    EMQC_TM_ARR_STR,
    EMQC_TM_ARR_DOUBLE,
    EMQC_TM_ARR_STRUCT,
    EMQC_TM_TERMINATOR,
} emqc_tm_type;

typedef struct {
    emqc_tm_type    type;
    char            *key;
    union {
        int         val_i;
        double      val_d;
        char        *val_s;
        void        *val_o;
    };

} emqc_tm_object;

typedef struct tm_ctxt tm_ctxt;

typedef struct {
    char *id;
    char *data;
    char *msg;
    int   code;
} tm_reply;

typedef struct {
    char *id;
    char *version;
    char *method;
    char *params;
} tm_request;

typedef struct {
    enum {TM_REP, TM_REQ, TM_RAW} type;
    union {
        tm_reply *rep;
        tm_request *req;
        char     *raw;
    };
} emqc_tm_trans_object;

typedef struct {
    emqc *    handle;
    tm_ctxt **ctxts;
} emqc_tm;

typedef struct {
    char *key;
    char *version;
} del_des_param;

emqc_tm *emqc_tm_new(void);

void emqc_tm_set_dft_cb(emqc_tm *tm, emqc_tm_cb_opt opt, emqc_cb cb);
void emqc_tm_set_ctm_cb(emqc_tm *tm, emqc_tm_cb_opt opt, emqc_cb cb, char *mid,
                        char *identifier);

emqc_tm *emqc_tm_init(emqc_tm *tm, const char *host, int port);

EMQC_ERR_CODE emqc_tm_connect(emqc_tm *tm);

void emqc_tm_destroy(emqc_tm *tm);

EMQC_ERR_CODE emqc_tm_sub(emqc_tm *tm);

EMQC_ERR_CODE emqc_tm_pub(emqc_tm *tm, const char *topic, const char *payload);

json_object *emqc_tm_new_payload(const char *mtd_dft, const char *func_id);

json_object *emqc_tm_add_get_desired_param(json_object *jso, const char **keys,
                                           int num_keys, char *func_id);

json_object *emqc_tm_add_del_desired_param(json_object *  jso,
                                           del_des_param *param_arr,
                                           int num_params, char *func_id);

json_object *emqc_tm_add_post_param(json_object *jso, emqc_tm_object *obj, char *func_id);

void emqc_tm_add_service_param(json_object *param, char *key, emqc_tm_object *obj);

json_object *emqc_tm_add_event_param(json_object *jso, emqc_tm_object *obj, bool ts);

void emqc_tm_property_set(json_object *jso_tmp, emqc_tm_object property_array[], char *model_id);

#endif
