#include <gtest/gtest.h>
#include <string>
using namespace std;

extern "C" {
#include <device_shadow.h>
}

TEST(device_shadow, new_request)
{
    serv_req *   re_test = new_request();
    bool         del_rep = get_del_rep(re_test);
    bool         del_des = get_del_des(re_test);
    long         version = get_version_req(re_test);
    json_object *ret     = get_json_object(re_test);
    json_object *expect  = json_object_new_object();

    EXPECT_FALSE(del_rep);
    EXPECT_FALSE(del_des);
    EXPECT_EQ(0, version);
    EXPECT_EQ(1, json_object_equal(ret, expect));
}

TEST(device_shadow, add_param_int)
{
    serv_req *   re_test = new_request();
    string       key1("attribute1");
    int          val1 = 1;
    string       comp1("{ \"reported\": { \"attribute1\":  1 } }");
    json_object *expect1 = json_tokener_parse(comp1.data());

    add_param_int(re_test, key1.data(), val1, REPORTED);
    json_object *ret1 = get_json_object(re_test);
    EXPECT_EQ(1, json_object_equal(ret1, expect1));

    string key2("attribute2");
    int    val2 = 2;
    string comp2("{ \"reported\": { \"attribute1\": 1, \"attribute2\": 2 } }");
    json_object *expect2 = json_tokener_parse(comp2.data());

    add_param_int(re_test, key2.data(), val2, REPORTED);
    json_object *ret2 = get_json_object(re_test);
    EXPECT_EQ(1, json_object_equal(ret2, expect2));
    // log_info("[payload]: %s",  json_object_to_json_string(ret2));
    // log_info("[payload]: %s",  json_object_to_json_string(expect2));
}

TEST(device_shadow, add_param_str)
{

    serv_req *   re_test = new_request();
    string       key1("attribute1");
    string       val1("emq");
    string       comp1("{ \"reported\": { \"attribute1\": \"emq\" } }");
    json_object *expect1 = json_tokener_parse(comp1.data());

    add_param_str(re_test, key1.data(), val1.data(), REPORTED);
    json_object *ret1 = get_json_object(re_test);
    EXPECT_EQ(1, json_object_equal(ret1, expect1));

    string key2("attribute2");
    string val2("emqx");
    string comp2("{ \"reported\": { \"attribute1\": \"emq\", \"attribute2\": "
                 "\"emqx\" } }");
    json_object *expect2 = json_tokener_parse(comp2.data());

    add_param_str(re_test, key2.data(), val2.data(), REPORTED);
    json_object *ret2 = get_json_object(re_test);
    EXPECT_EQ(1, json_object_equal(ret2, expect2));
    // log_info("[payload]: %s",  json_object_to_json_string(ret2));
    // log_info("[payload]: %s",  json_object_to_json_string(expect2));
}

TEST(device_shadow, add_param_arr)
{
    serv_req *   re_test = new_request();
    string       key1("attribute1");
    int          val1[] = { 1, 2, 3, 4, 5 };
    string       comp1("{ \"reported\": { \"attribute1\": [1,2,3,4,5] } }");
    json_object *expect1 = json_tokener_parse(comp1.data());

    add_param_arr(re_test, key1.data(), val1, INT, 5, REPORTED);
    json_object *ret1 = get_json_object(re_test);
    EXPECT_EQ(1, json_object_equal(ret1, expect1));

    string       key2("attribute2");
    int          val2[] = { 2, 2, 3, 4, 5 };
    string       comp2("{ \"reported\": { \"attribute1\": [1,2,3,4,5] , "
                 "\"attribute2\": [2,2,3,4,5] } }");
    json_object *expect2 = json_tokener_parse(comp2.data());

    add_param_arr(re_test, key2.data(), val2, INT, 5, REPORTED);
    json_object *ret2 = get_json_object(re_test);
    EXPECT_EQ(1, json_object_equal(ret2, expect2));
    // log_info("[payload]: %s",  json_object_to_json_string(ret2));
    // log_info("[payload]: %s",  json_object_to_json_string(expect2));
}

TEST(device_shadow, add_param_null)
{
    serv_req *   re_test = new_request();
    string       key("attribute1");
    string       val("emq");
    string       comp1("{ \"reported\": { \"attribute1\": \"emq\" } }");
    json_object *expect1 = json_tokener_parse(comp1.data());

    add_param_str(re_test, key.data(), val.data(), REPORTED);
    json_object *ret1 = get_json_object(re_test);
    EXPECT_EQ(1, json_object_equal(ret1, expect1));

    string       comp2("{ \"reported\": { \"attribute1\": \"null\" } }");
    json_object *expect2 = json_tokener_parse(comp2.data());

    add_param_null(re_test, key.data(), REPORTED);
    json_object *ret2 = get_json_object(re_test);

    EXPECT_EQ(1, json_object_equal(ret2, expect2));
    // log_info("[payload]: %s",  json_object_to_json_string(ret2));
    // log_info("[payload]: %s",  json_object_to_json_string(expect2));
}

TEST(device_shadow, add_param_null_reported)
{
    serv_req *   re_test = new_request();
    string       key("attribute1");
    string       val("emq");
    string       comp("{ \"reported\": { \"attribute1\": \"emq\" } }");
    json_object *expect = json_tokener_parse(comp.data());

    add_param_str(re_test, key.data(), val.data(), REPORTED);
    json_object *ret1 = get_json_object(re_test);
    EXPECT_EQ(1, json_object_equal(ret1, expect));

    add_param_null(re_test, NULL, REPORTED);
    json_object *ret2 = get_json_object(re_test);

    bool del_rep = get_del_rep(re_test);
    bool del_des = get_del_des(re_test);

    EXPECT_TRUE(del_rep);
    EXPECT_FALSE(del_des);

    EXPECT_EQ(1, json_object_equal(ret2, expect));

    // log_info("[payload]: %s",  json_object_to_json_string(ret2));
    // log_info("[payload]: %s",  json_object_to_json_string(expect));
}

TEST(device_shadow, get_all_param_attr)
{
    string       reply("{\"status\": \"success\"}");
    long         timestamp = 1614485144;
    json_object *payload   = json_tokener_parse(reply.data());
    serv_resp *  re_test   = new_response("reply", 1, 1614485144, payload);
    attr **      v         = get_all_param_attr(re_test, REPORTED);

    EXPECT_STREQ(v[0]->key, "status");
    EXPECT_STREQ(v[0]->val_str, "success");
    EXPECT_EQ(timestamp, v[0]->timestamp);

    EXPECT_EQ(get_version(re_test), 1);
}
