#include <gtest/gtest.h>
#include <string>
using namespace std;

extern "C" {
#include <thing_model.h>
}

emqc_tm *    etm = NULL;
json_object *jso = NULL;

TEST(thing_model, emqc_tm_new)
{
    ASSERT_TRUE(etm == NULL) << "tm is not NULL";
    etm = emqc_tm_new();
    ASSERT_TRUE(etm != NULL) << "tm is NULL";
    ASSERT_TRUE(etm->handle != NULL) << "tm handle is NULL";
    ASSERT_TRUE(etm->ctxts == NULL) << "tm ctxts is not NULL";
}

TEST(thing_model, emqc_tm_set_dft_cb)
{
    // emqc_tm_set_dft_cb(tm, TM_PROPERTY_SET_CB, property_set_cb);
}

TEST(thing_model, emqc_tm_set_ctm_cb) { }

TEST(thing_model, emqc_tm_destroy)
{
    ASSERT_TRUE(etm != NULL) << "tm is NULL";
    emqc_tm_destroy(etm);
    // ASSERT_TRUE(tm == NULL) << "tm is not NULL";
}

TEST(thing_model, emqc_tm_new_payload)
{
    ASSERT_TRUE(jso == NULL) << "json object is not NULL!";
    jso = emqc_tm_new_payload("t.p.p", NULL);
    string expect_s(
        "{ \"id\": \"0\", \"version\": \"1.0\", \"method\": \"t.p.p\" }");
    // cout << json_object_to_json_string(jso) << endl;
    json_object *jso_expect = json_tokener_parse(expect_s.data());
    ASSERT_TRUE(jso != NULL) << "json object is NULL!";
    ASSERT_TRUE(jso_expect != NULL) << "json object is NULL!";
    ASSERT_EQ(1, json_object_equal(jso, jso_expect));
}

TEST(thing_model, emqc_tm_add_post_param)
{
    ASSERT_TRUE(jso != NULL) << "json object is NULL!";

    emqc_tm_object tm_object =  {.type = EMQC_TM_INT,  .key = (char*) "Power", .val_i = 10 };

    jso = emqc_tm_add_post_param(jso, &tm_object, NULL);
    string       expect_s("{ \"id\": \"0\", \"version\": \"1.0\", \"method\": "
                    "\"t.p.p\", \"params\": { \"Power\": { \"value\": 10} } }");
    json_object *jso_expect = json_tokener_parse(expect_s.data());
    ASSERT_TRUE(jso != NULL) << "json object is NULL!";
    ASSERT_TRUE(jso_expect != NULL) << "json object is NULL!";
    json_object *jso_p   = json_object_object_get(jso, "params");
    json_object *jsoe_p  = json_object_object_get(jso_expect, "params");
    json_object *jso_pp  = json_object_object_get(jso_p, "Power");
    json_object *jsoe_pp = json_object_object_get(jsoe_p, "Power");
    json_object_object_add(jsoe_pp, "timestamp",
                           json_object_object_get(jso_pp, "timestamp"));
    // cout << json_object_to_json_string(jso) << endl;

    ASSERT_EQ(1, json_object_equal(jso, jso_expect));
}

TEST(thing_model, emqc_tm_add_event_param)
{
    ASSERT_TRUE(jso != NULL) << "json object is NULL!";
    int val = 10;
    json_object_object_del(jso, "params");
    emqc_tm_object tm_object =  {.type = EMQC_TM_INT,  .key = (char*) "Power", .val_i = 10 };
    jso = emqc_tm_add_event_param(jso, &tm_object, false);
    string       expect_s("{ \"id\": \"0\", \"version\": \"1.0\", \"method\": "
                    "\"t.p.p\", \"params\": { \"Power\": 10 } }");
    json_object *jso_expect = json_tokener_parse(expect_s.data());
    ASSERT_TRUE(jso != NULL) << "json object is NULL!";
    ASSERT_TRUE(jso_expect != NULL) << "json object is NULL!";
    // cout << json_object_to_json_string(jso) << endl;

    ASSERT_EQ(1, json_object_equal(jso, jso_expect));
}

TEST(thing_model, emqc_tm_add_del_desired_param)
{
    ASSERT_TRUE(jso != NULL) << "json object is NULL!";
    del_des_param ddp[] = { { (char *) "WF", NULL },
                            { (char *) "Power", (char *) "1" } };
    json_object_object_del(jso, "params");
    emqc_tm_add_del_desired_param(jso, ddp, 2, NULL);
    string expect_s(
        "{ \"id\": \"0\", \"version\": \"1.0\", \"method\": \"t.p.p\", "
        "\"params\": { \"WF\": { }, \"Power\": { \"version\": \"1\" } } }");
    json_object *jso_expect = json_tokener_parse(expect_s.data());
    ASSERT_TRUE(jso != NULL) << "json object is NULL!";
    ASSERT_TRUE(jso_expect != NULL) << "json object is NULL!";

    // cout << json_object_to_json_string(jso) << endl;

    ASSERT_EQ(1, json_object_equal(jso, jso_expect));
}

TEST(thing_model, emqc_tm_add_get_desired_param)
{
    ASSERT_TRUE(jso != NULL) << "json object is NULL!";
    const char *keys[] = { "WF", "Power" };
    json_object_object_del(jso, "params");
    emqc_tm_add_get_desired_param(jso, (const char **) keys, 2, NULL);
    string       expect_s("{ \"id\": \"0\", \"version\": \"1.0\", \"method\": "
                    "\"t.p.p\", \"params\": [ \"WF\", \"Power\"] }");
    json_object *jso_expect = json_tokener_parse(expect_s.data());
    ASSERT_TRUE(jso != NULL) << "json object is NULL!";
    ASSERT_TRUE(jso_expect != NULL) << "json object is NULL!";

    // cout << json_object_to_json_string(jso) << endl;

    ASSERT_EQ(1, json_object_equal(jso, jso_expect));
}
